/*
 *  Copyright (C) 2008 Florent Bedoiseau (electronique.fb@free.fr)
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <string.h>
#include <stdio.h>
#include <iostream>

#include "AWD.h"
#include "AWA.h"

static int AWDId=0;

AWD::AWD (int ax, int ay, int aw, int ah, const char *atitle) : AW (0,ax,ay,aw,ah) {

  Id=AWDId;
  AWDId++;
  char buf [64];
  sprintf (buf, "(%d)", Id);

  if (atitle) {
    title=new char [strlen(atitle)+65];
#if DEBUG_OVERLAP
    sprintf (title, "%s %s", buf, atitle);
#else
    sprintf (title, "%s", atitle);
#endif
  }
  else {
    title=0;
  }
  mAWSelected=0;
  saved_area=GfxCreateBitmap(w,h);
  needToHide=false;
  needToShow=false;
}

AWD::~AWD () {
  // The title
  if (title) delete [] title;

  // The children
  set<AW *>::iterator it = setOfAW.begin ();
  for (; it != setOfAW.end(); it++) {
    delete (*it);
  }

  if (saved_area) GfxDeleteBitmap(saved_area);
}

bool AWD::AttachObject (AW *aAW) {
  if (aAW) {
    setOfAW.insert (setOfAW.end(), aAW);
    return true; //OK
  }
  return false; // KO
}

AW* AWD::FindAWUnderMouse (int mx,int my) {
 set<AW *>::iterator it = setOfAW.begin ();
  for (; it != setOfAW.end(); it++) {
    if ((*it)->IsInside (mx,my)==1) { // Only the one under the mouse
      if ((*it)->IsVisible()) return (*it); // only if visible
    }
  }
  return 0; // None found
}

bool AWD::ProcessEvent (int mouse_move, UInt16 mx, UInt16 my,
						int mouse_button, bool mb1, bool mb2, EVT & evt) {

  bool status = false;
  // Entering or leaving an Object
  // Only when mouse button 1 is not pressed
  if (mb1==false) {
    AW *aAW=FindAWUnderMouse (mx,my);
    if (aAW) {
      if (aAW!=mAWSelected) {
		if (mAWSelected) {
		  evt = mAWSelected->Leave ();
		  if (evt.GetType() != EVT_NONE) status = true;
		}
		aAW->Enter ();
		mAWSelected=aAW;
      }
    }
    else {
      if (mAWSelected) {
		evt = mAWSelected->Leave ();
		if (evt.GetType() != EVT_NONE) status = true;
		mAWSelected=0;
      }
    }
  }

  if (mAWSelected) {
	// Send the message to the corresponding object under the cursor
	return mAWSelected->ProcessEvent (mouse_move, mx, my, mouse_button, mb1, mb2, evt);
  }

  // Otherwise we move the dialog
  static int xx,yy;
  if (mb1==true) {
    if (mouse_move) {
      Move (x+(mx-xx), y+(my-yy));
      xx=mx;
      yy=my;
    }
    else {
      xx=mx;
      yy=my;
    }
  }

  return status;
}

int AWD::Move (int ax, int ay) {
  int ox,oy;
  GetXY (ox,oy); // Old x,y
  if (AW::Move (ax,ay) == 1) return 1;  // Nothing to do
  GetXY (ax,ay); // This not perhaps the x,y we wanted. We take the current ones

  // Now move all children here
  set<AW *>::iterator it = setOfAW.begin ();
  for (; it != setOfAW.end(); it++) {
    int x,y;
    (*it)->GetXY (x,y); // Old x,y of a child
    (*it)->Move (ax+(x-ox),ay+(y-oy));
  }
  return 0;
}

void AWD::InternalShow() {
  AW::Show ();

  // Now move all children here
  set<AW *>::iterator it = setOfAW.begin ();
  for (; it != setOfAW.end(); it++) {
    (*it)->Show ();
  }
}

void AWD::InternalHide() {
  AW::Hide ();

  // Now move all children here
  set<AW *>::iterator it = setOfAW.begin ();
   for (; it != setOfAW.end(); it++) {
     (*it)->Hide ();
  }
}

void AWD::Show () {
    if (IsVisible()) return;
    if (!needToShow) needToShow=true;
}

void AWD::Hide () {
    if (!IsVisible()) return; // Already hidden
    if (!needToHide) needToHide=true;
}

void AWD::Draw () {
  // Copy the area before drawing
  GfxBlit (0,saved_area,x,y,0,0,w,h);

  // Draw itself, then all the objects
  GfxFillRectangle(0,x,y,w,h,foreground_color); // All the area

  // Borders
  GfxLine(0,x,y,x+w-1,y, shadow_color_light);
  GfxLine(0,x,y+h-1,x+w-1,y+h-1, shadow_color_dark);
  GfxLine(0,x,y,x,y+h-1, shadow_color_light);
  GfxLine(0,x+w-1,y,x+w-1,y+h-1, shadow_color_dark);

  // Bandeau
  GfxFillRectangle(0,x+2,y+2,w-4,14,background_color); // All the area
  GfxSetClip(0,x,y,x+w-1,y+h-1);
  GfxStringXY (0, title, x+10, y+6, foreground_color, background_color);
  GfxSetClip (0,0,0,0,0);
}

void AWD::UnDraw () {
  GfxBlit (saved_area,0,0,0,x,y,w,h);
}

EVT AWD::Leave () {
#if DEBUG_EVT
  printf ("AWD::Leave\n");
#endif
  EVT event;

  // OK, leaving the object also ...
  if (mAWSelected) {
	event = mAWSelected->Leave ();
    mAWSelected=0;
  }
  return event;
}
void AWD::Enter () {
#if DEBUG_EVT
  printf ("AWD::Enter\n");
#endif
}

// End of file
